@@ -31,6 +31,8 @@
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
+from __future__ import annotations
+
 import copy
 import csv
 import io
@@ -41,8 +43,9 @@ import re
 import textwrap
 from html import escape
 from html.parser import HTMLParser
+from typing import Any
 
-import wcwidth
+import wcwidth  # type: ignore
 
 # hrule styles
 FRAME = 0
@@ -59,6 +62,7 @@ ORGMODE = 14
 DOUBLE_BORDER = 15
 SINGLE_BORDER = 16
 RANDOM = 20
+BASE_ALIGN_VALUE = "base_align_value"
 
 _re = re.compile(r"\033\[[0-9;]*m|\033\(B")
 
@@ -71,7 +75,7 @@ def _get_size(text):
 
 
 class PrettyTable:
-    def __init__(self, field_names=None, **kwargs):
+    def __init__(self, field_names=None, **kwargs) -> None:
         """Return a new PrettyTable instance
 
         Arguments:
@@ -86,6 +90,8 @@ class PrettyTable:
         header_style - stylisation to apply to field names in header
             ("cap", "title", "upper", "lower" or None)
         border - print a border around the table (True or False)
+        preserve_internal_border - print a border inside the table even if
+            border is disabled (True or False)
         hrules - controls printing of horizontal rules after rows.
             Allowed values: FRAME, HEADER, ALL, NONE
         vrules - controls printing of vertical rules between columns.
@@ -128,8 +134,9 @@ class PrettyTable:
         self.encoding = kwargs.get("encoding", "UTF-8")
 
         # Data
-        self._field_names = []
-        self._rows = []
+        self._field_names: list[str] = []
+        self._rows: list[list] = []
+        self._dividers: list[bool] = []
         self.align = {}
         self.valign = {}
         self.max_width = {}
@@ -141,7 +148,7 @@ class PrettyTable:
         if field_names:
             self.field_names = field_names
         else:
-            self._widths = []
+            self._widths: list[int] = []
 
         # Options
         self._options = [
@@ -151,6 +158,7 @@ class PrettyTable:
             "fields",
             "header",
             "border",
+            "preserve_internal_border",
             "sortby",
             "reversesort",
             "sort_key",
@@ -199,7 +207,7 @@ class PrettyTable:
         self._start = kwargs["start"] or 0
         self._end = kwargs["end"] or None
         self._fields = kwargs["fields"] or None
-        self._none_format = {}
+        self._none_format: dict[None, None] = {}
 
         if kwargs["header"] in (True, False):
             self._header = kwargs["header"]
@@ -210,6 +218,10 @@ class PrettyTable:
             self._border = kwargs["border"]
         else:
             self._border = True
+        if kwargs["preserve_internal_border"] in (True, False):
+            self._preserve_internal_border = kwargs["preserve_internal_border"]
+        else:
+            self._preserve_internal_border = False
         self._hrules = kwargs["hrules"] or FRAME
         self._vrules = kwargs["vrules"] or ALL
 
@@ -286,7 +298,6 @@ class PrettyTable:
                 return (excess // 2) * " " + text + (excess // 2) * " "
 
     def __getattr__(self, name):
-
         if name == "rowcount":
             return len(self._rows)
         elif name == "colcount":
@@ -300,7 +311,6 @@ class PrettyTable:
             raise AttributeError(name)
 
     def __getitem__(self, index):
-
         new = PrettyTable()
         new.field_names = self.field_names
         for attr in self._options:
@@ -372,6 +382,7 @@ class PrettyTable:
         elif option in (
             "header",
             "border",
+            "preserve_internal_border",
             "reversesort",
             "xhtml",
             "print_empty",
@@ -549,11 +560,15 @@ class PrettyTable:
     # ATTRIBUTE MANAGEMENT       #
     ##############################
     @property
-    def rows(self):
+    def rows(self) -> list[Any]:
         return self._rows[:]
 
     @property
-    def xhtml(self):
+    def dividers(self) -> list[bool]:
+        return self._dividers[:]
+
+    @property
+    def xhtml(self) -> bool:
         """Print <br/> tags if True, <br> tags if False"""
         return self._xhtml
 
@@ -601,6 +616,9 @@ class PrettyTable:
             for old_name in old_names:
                 if old_name not in self._align:
                     self._align.pop(old_name)
+        elif self._align:
+            for field_name in self._field_names:
+                self._align[field_name] = self._align[BASE_ALIGN_VALUE]
         else:
             self.align = "c"
         if self._valign and old_names:
@@ -622,15 +640,19 @@ class PrettyTable:
 
     @align.setter
     def align(self, val):
-        if not self._field_names:
-            self._align = {}
-        elif val is None or (isinstance(val, dict) and len(val) == 0):
-            for field in self._field_names:
-                self._align[field] = "c"
+        if val is None or (isinstance(val, dict) and len(val) == 0):
+            if not self._field_names:
+                self._align = {BASE_ALIGN_VALUE: "c"}
+            else:
+                for field in self._field_names:
+                    self._align[field] = "c"
         else:
             self._validate_align(val)
-            for field in self._field_names:
-                self._align[field] = val
+            if not self._field_names:
+                self._align = {BASE_ALIGN_VALUE: val}
+            else:
+                for field in self._field_names:
+                    self._align[field] = val
 
     @property
     def valign(self):
@@ -842,6 +864,21 @@ class PrettyTable:
         self._validate_option("border", val)
         self._border = val
 
+    @property
+    def preserve_internal_border(self):
+        """Controls printing of border inside table
+
+        Arguments:
+
+        preserve_internal_border - print a border inside the table even if
+            border is disabled (True or False)"""
+        return self._preserve_internal_border
+
+    @preserve_internal_border.setter
+    def preserve_internal_border(self, val):
+        self._validate_option("preserve_internal_border", val)
+        self._preserve_internal_border = val
+
     @property
     def hrules(self):
         """Controls printing of horizontal rules after rows
@@ -1095,7 +1132,8 @@ class PrettyTable:
 
     @property
     def top_right_junction_char(self):
-        """The character used when printing table borders to draw top-right line junctions
+        """
+        The character used when printing table borders to draw top-right line junctions
 
         Arguments:
 
@@ -1219,7 +1257,6 @@ class PrettyTable:
     ##############################
 
     def _get_options(self, kwargs):
-
         options = {}
         for option in self._options:
             if option in kwargs:
@@ -1233,8 +1270,7 @@ class PrettyTable:
     # PRESET STYLE LOGIC         #
     ##############################
 
-    def set_style(self, style):
-
+    def set_style(self, style) -> None:
         if style == DEFAULT:
             self._set_default_style()
         elif style == MSWORD_FRIENDLY:
@@ -1270,7 +1306,6 @@ class PrettyTable:
         self._horizontal_align_char = ":"
 
     def _set_default_style(self):
-
         self.header = True
         self.border = True
         self._hrules = FRAME
@@ -1292,7 +1327,6 @@ class PrettyTable:
         self._bottom_left_junction_char = None
 
     def _set_msword_style(self):
-
         self.header = True
         self.border = True
         self._hrules = NONE
@@ -1302,7 +1336,6 @@ class PrettyTable:
         self.vertical_char = "|"
 
     def _set_columns_style(self):
-
         self.header = True
         self.border = False
         self.padding_width = 1
@@ -1336,7 +1369,6 @@ class PrettyTable:
         self.bottom_left_junction_char = "└"
 
     def _set_random_style(self):
-
         # Just for fun!
         self.header = random.choice((True, False))
         self.border = random.choice((True, False))
@@ -1347,13 +1379,13 @@ class PrettyTable:
         self.vertical_char = random.choice(r"~!@#$%^&*()_+|-=\{}[];':\",./;<>?")
         self.horizontal_char = random.choice(r"~!@#$%^&*()_+|-=\{}[];':\",./;<>?")
         self.junction_char = random.choice(r"~!@#$%^&*()_+|-=\{}[];':\",./;<>?")
+        self.preserve_internal_border = random.choice((True, False))
 
     ##############################
     # DATA INPUT METHODS         #
     ##############################
 
-    def add_rows(self, rows):
-
+    def add_rows(self, rows) -> None:
         """Add rows to the table
 
         Arguments:
@@ -1363,8 +1395,7 @@ class PrettyTable:
         for row in rows:
             self.add_row(row)
 
-    def add_row(self, row):
-
+    def add_row(self, row, *, divider=False) -> None:
         """Add a row to the table
 
         Arguments:
@@ -1380,9 +1411,9 @@ class PrettyTable:
         if not self._field_names:
             self.field_names = [f"Field {n + 1}" for n in range(0, len(row))]
         self._rows.append(list(row))
+        self._dividers.append(divider)
 
-    def del_row(self, row_index):
-
+    def del_row(self, row_index) -> None:
         """Delete a row from the table
 
         Arguments:
@@ -1395,9 +1426,11 @@ class PrettyTable:
                 f"table only has {len(self._rows)} rows"
             )
         del self._rows[row_index]
+        del self._dividers[row_index]
 
-    def add_column(self, fieldname, column, align="c", valign="t"):
-
+    def add_column(
+        self, fieldname, column, align: str = "c", valign: str = "t"
+    ) -> None:
         """Add a column to the table.
 
         Arguments:
@@ -1419,6 +1452,7 @@ class PrettyTable:
             for i in range(0, len(column)):
                 if len(self._rows) < i + 1:
                     self._rows.append([])
+                    self._dividers.append(False)
                 self._rows[i].append(column[i])
         else:
             raise ValueError(
@@ -1426,7 +1460,7 @@ class PrettyTable:
                 f"{len(self._rows)}"
             )
 
-    def add_autoindex(self, fieldname="Index"):
+    def add_autoindex(self, fieldname: str = "Index"):
         """Add an auto-incrementing index column to the table.
         Arguments:
         fieldname - name of the field to contain the new column of data"""
@@ -1436,8 +1470,7 @@ class PrettyTable:
         for i, row in enumerate(self._rows):
             row.insert(0, i + 1)
 
-    def del_column(self, fieldname):
-
+    def del_column(self, fieldname) -> None:
         """Delete a column from the table
 
         Arguments:
@@ -1456,18 +1489,18 @@ class PrettyTable:
         for row in self._rows:
             del row[col_index]
 
-    def clear_rows(self):
-
+    def clear_rows(self) -> None:
         """Delete all rows from the table but keep the current field names"""
 
         self._rows = []
+        self._dividers = []
 
-    def clear(self):
-
+    def clear(self) -> None:
         """Delete all rows and field names from the table, maintaining nothing but
         styling options"""
 
         self._rows = []
+        self._dividers = []
         self._field_names = []
         self._widths = []
 
@@ -1478,6 +1511,28 @@ class PrettyTable:
     def copy(self):
         return copy.deepcopy(self)
 
+    def get_formatted_string(self, out_format: str = "text", **kwargs) -> str:
+        """Return string representation of specified format of table in current state.
+
+        Arguments:
+        out_format - resulting table format
+        kwargs - passed through to function that performs formatting
+        """
+        if out_format == "text":
+            return self.get_string(**kwargs)
+        if out_format == "html":
+            return self.get_html_string(**kwargs)
+        if out_format == "json":
+            return self.get_json_string(**kwargs)
+        if out_format == "csv":
+            return self.get_csv_string(**kwargs)
+        if out_format == "latex":
+            return self.get_latex_string(**kwargs)
+        raise ValueError(
+            f"Invalid format {out_format}. "
+            "Must be one of: text, html, json, csv, or latex"
+        )
+
     ##############################
     # MISC PRIVATE METHODS       #
     ##############################
@@ -1545,15 +1600,29 @@ class PrettyTable:
                 title_width = 0
             min_table_width = self.min_table_width or 0
             min_width = max(title_width, min_table_width)
-            table_width = self._compute_table_width(options)
-            if table_width < min_width:
+            if options["border"]:
+                borders = len(widths) + 1
+            elif options["preserve_internal_border"]:
+                borders = len(widths)
+            else:
+                borders = 0
+
+            # Subtract padding for each column and borders
+            min_width -= (
+                sum([sum(self._get_padding_widths(options)) for _ in widths]) + borders
+            )
+            # What is being scaled is content so we sum column widths
+            content_width = sum(widths) or 1
+
+            if content_width < min_width:
                 # Grow widths in proportion
-                scale = 1.0 * min_width / table_width
-                widths = [int(math.ceil(w * scale)) for w in widths]
+                scale = 1.0 * min_width / content_width
+                widths = [int(math.floor(w * scale)) for w in widths]
+                if sum(widths) < min_width:
+                    widths[-1] += min_width - sum(widths)
                 self._widths = widths
 
     def _get_padding_widths(self, options):
-
         if options["left_padding_width"] is not None:
             lpad = options["left_padding_width"]
         else:
@@ -1593,6 +1662,23 @@ class PrettyTable:
 
         return rows
 
+    def _get_dividers(self, options):
+        """Return only those dividers that should be printed, based on slicing.
+
+        Arguments:
+
+        options - dictionary of option settings."""
+
+        if options["oldsortslice"]:
+            dividers = copy.deepcopy(self._dividers[options["start"] : options["end"]])
+        else:
+            dividers = copy.deepcopy(self._dividers)
+
+        if options["sortby"]:
+            dividers = [False for divider in dividers]
+
+        return dividers
+
     def _format_row(self, row):
         return [
             self._format_value(field, value)
@@ -1606,8 +1692,7 @@ class PrettyTable:
     # PLAIN TEXT STRING METHODS  #
     ##############################
 
-    def get_string(self, **kwargs):
-
+    def get_string(self, **kwargs) -> str:
         """Return string representation of table in current state.
 
         Arguments:
@@ -1618,6 +1703,8 @@ class PrettyTable:
         fields - names of fields (columns) to include
         header - print a header showing field names (True or False)
         border - print a border around the table (True or False)
+        preserve_internal_border - print a border inside the table even if
+            border is disabled (True or False)
         hrules - controls printing of horizontal rules after rows.
             Allowed values: ALL, FRAME, HEADER, NONE
         vrules - controls printing of vertical rules between columns.
@@ -1664,6 +1751,7 @@ class PrettyTable:
 
         # Get the rows we need to print, taking into account slicing, sorting, etc.
         rows = self._get_rows(options)
+        dividers = self._get_dividers(options)
 
         # Turn all data in all rows into Unicode, formatted as desired
         formatted_rows = self._format_rows(rows)
@@ -1688,8 +1776,10 @@ class PrettyTable:
                 )
 
         # Add rows
-        for row in formatted_rows[:-1]:
+        for row, divider in zip(formatted_rows[:-1], dividers[:-1]):
             lines.append(self._stringify_row(row, options, self._hrule))
+            if divider:
+                lines.append(self._stringify_hrule(options, where="bottom_"))
         if formatted_rows:
             lines.append(
                 self._stringify_row(
@@ -1712,8 +1802,7 @@ class PrettyTable:
         return "\n".join(lines)
 
     def _stringify_hrule(self, options, where=""):
-
-        if not options["border"]:
+        if not options["border"] and not options["preserve_internal_border"]:
             return ""
         lpad, rpad = self._get_padding_widths(options)
         if options["vrules"] in (ALL, FRAME):
@@ -1745,10 +1834,13 @@ class PrettyTable:
         if options["vrules"] in (ALL, FRAME):
             bits.pop()
             bits.append(options[where + "right_junction_char"])
+
+        if options["preserve_internal_border"] and not options["border"]:
+            bits = bits[1:-1]
+
         return "".join(bits)
 
     def _stringify_title(self, title, options):
-
         lines = []
         lpad, rpad = self._get_padding_widths(options)
         if options["border"]:
@@ -1760,7 +1852,9 @@ class PrettyTable:
                 lines.append(self._stringify_hrule(options, "top_"))
         bits = []
         endpoint = (
-            options["vertical_char"] if options["vrules"] in (ALL, FRAME) else " "
+            options["vertical_char"]
+            if options["vrules"] in (ALL, FRAME) and options["border"]
+            else " "
         )
         bits.append(endpoint)
         title = " " * lpad + title + " " * rpad
@@ -1770,7 +1864,6 @@ class PrettyTable:
         return "\n".join(lines)
 
     def _stringify_header(self, options):
-
         bits = []
         lpad, rpad = self._get_padding_widths(options)
         if options["border"]:
@@ -1793,7 +1886,7 @@ class PrettyTable:
                 bits.append(options["vertical_char"])
             else:
                 bits.append(" ")
-        for (field, width) in zip(self._field_names, self._widths):
+        for field, width in zip(self._field_names, self._widths):
             if options["fields"] and field not in options["fields"]:
                 continue
             if self._header_style == "cap":
@@ -1813,24 +1906,31 @@ class PrettyTable:
                 + self._justify(fieldname, width, self._align[field])
                 + " " * rpad
             )
-            if options["border"]:
+            if options["border"] or options["preserve_internal_border"]:
                 if options["vrules"] == ALL:
                     bits.append(options["vertical_char"])
                 else:
                     bits.append(" ")
+
+        # If only preserve_internal_border is true, then we just appended
+        # a vertical character at the end when we wanted a space
+        if not options["border"] and options["preserve_internal_border"]:
+            bits.pop()
+            bits.append(" ")
         # If vrules is FRAME, then we just appended a space at the end
         # of the last field, when we really want a vertical character
         if options["border"] and options["vrules"] == FRAME:
             bits.pop()
             bits.append(options["vertical_char"])
-        if options["border"] and options["hrules"] != NONE:
+        if (options["border"] or options["preserve_internal_border"]) and options[
+            "hrules"
+        ] != NONE:
             bits.append("\n")
             bits.append(self._hrule)
         return "".join(bits)
 
     def _stringify_row(self, row, options, hrule):
-
-        for (index, field, value, width) in zip(
+        for index, field, value, width in zip(
             range(0, len(row)), self._field_names, row, self._widths
         ):
             # Enforce max widths
@@ -1862,8 +1962,7 @@ class PrettyTable:
                 else:
                     bits[y].append(" ")
 
-        for (field, value, width) in zip(self._field_names, row, self._widths):
-
+        for field, value, width in zip(self._field_names, row, self._widths):
             valign = self._valign[field]
             lines = value.split("\n")
             d_height = row_height - len(lines)
@@ -1889,13 +1988,19 @@ class PrettyTable:
                     + self._justify(line, width, self._align[field])
                     + " " * rpad
                 )
-                if options["border"]:
+                if options["border"] or options["preserve_internal_border"]:
                     if options["vrules"] == ALL:
                         bits[y].append(self.vertical_char)
                     else:
                         bits[y].append(" ")
                 y += 1
 
+        # If only preserve_internal_border is true, then we just appended
+        # a vertical character at the end when we wanted a space
+        if not options["border"] and options["preserve_internal_border"]:
+            bits[-1].pop()
+            bits[-1].append(" ")
+
         # If vrules is FRAME, then we just appended a space at the end
         # of the last field, when we really want a vertical character
         for y in range(0, row_height):
@@ -1912,8 +2017,7 @@ class PrettyTable:
 
         return "\n".join(bits)
 
-    def paginate(self, page_length=58, **kwargs):
-
+    def paginate(self, page_length: int = 58, line_break: str = "\f", **kwargs):
         pages = []
         kwargs["start"] = kwargs.get("start", 0)
         true_end = kwargs.get("end", self.rowcount)
@@ -1923,13 +2027,12 @@ class PrettyTable:
             if kwargs["end"] == true_end:
                 break
             kwargs["start"] += page_length
-        return "\f".join(pages)
+        return line_break.join(pages)
 
     ##############################
     # CSV STRING METHODS         #
     ##############################
-    def get_csv_string(self, **kwargs):
-
+    def get_csv_string(self, **kwargs) -> str:
         """Return string representation of CSV formatted table in the current state
 
         Keyword arguments are first interpreted as table formatting options, and
@@ -1956,8 +2059,7 @@ class PrettyTable:
     ##############################
     # JSON STRING METHODS        #
     ##############################
-    def get_json_string(self, **kwargs):
-
+    def get_json_string(self, **kwargs) -> str:
         """Return string representation of JSON formatted table in the current state
 
         Keyword arguments are first interpreted as table formatting options, and
@@ -1968,7 +2070,7 @@ class PrettyTable:
         """
 
         options = self._get_options(kwargs)
-        json_options = dict(indent=4, separators=(",", ": "), sort_keys=True)
+        json_options: Any = dict(indent=4, separators=(",", ": "), sort_keys=True)
         json_options.update(
             {key: value for key, value in kwargs.items() if key not in options}
         )
@@ -1985,7 +2087,7 @@ class PrettyTable:
     # HTML STRING METHODS        #
     ##############################
 
-    def get_html_string(self, **kwargs):
+    def get_html_string(self, **kwargs) -> str:
         """Return string representation of HTML formatted version of table in current
         state.
 
@@ -1997,6 +2099,8 @@ class PrettyTable:
         fields - names of fields (columns) to include
         header - print a header showing field names (True or False)
         border - print a border around the table (True or False)
+        preserve_internal_border - print a border inside the table even if
+            border is disabled (True or False)
         hrules - controls printing of horizontal rules after rows.
             Allowed values: ALL, FRAME, HEADER, NONE
         vrules - controls printing of vertical rules between columns.
@@ -2026,7 +2130,6 @@ class PrettyTable:
         return string
 
     def _get_simple_html_string(self, options):
-
         lines = []
         if options["xhtml"]:
             linebreak = "<br/>"
@@ -2077,7 +2180,6 @@ class PrettyTable:
         return "\n".join(lines)
 
     def _get_formatted_html_string(self, options):
-
         lines = []
         lpad, rpad = self._get_padding_widths(options)
         if options["xhtml"]:
@@ -2101,6 +2203,8 @@ class PrettyTable:
                 open_tag.append(' frame="vsides"')
             elif options["vrules"] == ALL:
                 open_tag.append(' frame="vsides" rules="cols"')
+        if not options["border"] and options["preserve_internal_border"]:
+            open_tag.append(' rules="cols"')
         if options["attributes"]:
             for attr_name in options["attributes"]:
                 open_tag.append(f' {attr_name}="{options["attributes"][attr_name]}"')
@@ -2166,7 +2270,7 @@ class PrettyTable:
     # LATEX STRING METHODS       #
     ##############################
 
-    def get_latex_string(self, **kwargs):
+    def get_latex_string(self, **kwargs) -> str:
         """Return string representation of LaTex formatted version of table in current
         state.
 
@@ -2177,6 +2281,8 @@ class PrettyTable:
         fields - names of fields (columns) to include
         header - print a header showing field names (True or False)
         border - print a border around the table (True or False)
+        preserve_internal_border - print a border inside the table even if
+            border is disabled (True or False)
         hrules - controls printing of horizontal rules after rows.
             Allowed values: ALL, FRAME, HEADER, NONE
         vrules - controls printing of vertical rules between columns.
@@ -2243,6 +2349,8 @@ class PrettyTable:
         wanted_alignments = [self._align[field] for field in wanted_fields]
         if options["border"] and options["vrules"] == ALL:
             alignment_str = "|".join(wanted_alignments)
+        elif not options["border"] and options["preserve_internal_border"]:
+            alignment_str = "|".join(wanted_alignments)
         else:
             alignment_str = "".join(wanted_alignments)
 
@@ -2251,14 +2359,15 @@ class PrettyTable:
 
         begin_cmd = "\\begin{tabular}{%s}" % alignment_str
         lines.append(begin_cmd)
-
         if options["border"] and options["hrules"] in [ALL, FRAME]:
             lines.append("\\hline")
 
         # Headers
         if options["header"]:
             lines.append(" & ".join(wanted_fields) + " \\\\")
-        if options["border"] and options["hrules"] in [ALL, HEADER]:
+        if (options["border"] or options["preserve_internal_border"]) and options[
+            "hrules"
+        ] in [ALL, HEADER]:
             lines.append("\\hline")
 
         # Data
@@ -2295,7 +2404,7 @@ def _str_block_width(val):
 ##############################
 
 
-def from_csv(fp, field_names=None, **kwargs):
+def from_csv(fp, field_names: Any | None = None, **kwargs):
     fmtparams = {}
     for param in [
         "delimiter",
@@ -2348,27 +2457,27 @@ def from_json(json_string, **kwargs):
 
 
 class TableHandler(HTMLParser):
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs) -> None:
         HTMLParser.__init__(self)
         self.kwargs = kwargs
-        self.tables = []
-        self.last_row = []
-        self.rows = []
+        self.tables: list[list] = []
+        self.last_row: list[str] = []
+        self.rows: list[Any] = []
         self.max_row_width = 0
         self.active = None
         self.last_content = ""
         self.is_last_row_header = False
         self.colspan = 0
 
-    def handle_starttag(self, tag, attrs):
+    def handle_starttag(self, tag, attrs) -> None:
         self.active = tag
         if tag == "th":
             self.is_last_row_header = True
-        for (key, value) in attrs:
+        for key, value in attrs:
             if key == "colspan":
                 self.colspan = int(value)
 
-    def handle_endtag(self, tag):
+    def handle_endtag(self, tag) -> None:
         if tag in ["th", "td"]:
             stripped_content = self.last_content.strip()
             self.last_row.append(stripped_content)
@@ -2389,7 +2498,7 @@ class TableHandler(HTMLParser):
         self.last_content = " "
         self.active = None
 
-    def handle_data(self, data):
+    def handle_data(self, data) -> None:
         self.last_content += data
 
     def generate_table(self, rows):
@@ -2410,7 +2519,7 @@ class TableHandler(HTMLParser):
                 table.add_row(row[0])
         return table
 
-    def make_fields_unique(self, fields):
+    def make_fields_unique(self, fields) -> None:
         """
         iterates over the row and make each field unique
         """
