@@ -1,9 +1,11 @@
+from __future__ import annotations
+
 import datetime as dt
 import io
 import random
 import sqlite3
 from math import e, pi, sqrt
-from typing import Any, List
+from typing import Any
 
 import pytest
 
@@ -553,6 +555,13 @@ class TestBasic:
         city_data_prettytable.title = "My table"
         self._test_all_length_equal(city_data_prettytable)
 
+    def test_all_lengths_equal_with_long_title(
+        self, city_data_prettytable: PrettyTable
+    ):
+        """All lines in a table should be of the same length, even with a long title."""
+        city_data_prettytable.title = "My table (75 characters wide) " + "=" * 45
+        self._test_all_length_equal(city_data_prettytable)
+
     def test_no_blank_lines_without_border(self, city_data_prettytable: PrettyTable):
         """No table should ever have blank lines in it."""
         city_data_prettytable.border = False
@@ -861,7 +870,7 @@ class TestBreakLine:
         ],
     )
     def test_break_line_ASCII(
-        self, rows: List[List[Any]], hrule: int, expected_result: str
+        self, rows: list[list[Any]], hrule: int, expected_result: str
     ):
         t = PrettyTable(["Field 1", "Field 2"])
         for row in rows:
@@ -1729,7 +1738,6 @@ def test_add_rows():
 
 
 def test_autoindex():
-
     """Testing that a table with a custom index row is
     equal to the one produced by the function
     .add_autoindex()
@@ -1903,6 +1911,59 @@ class TestRepr:
         assert row_prettytable._repr_html_() == row_prettytable.get_html_string()
 
 
+class TestMinTableWidth:
+    @pytest.mark.parametrize(
+        "loops, fields, desired_width, border, internal_border",
+        [
+            (15, ["Test table"], 20, True, False),
+            (16, ["Test table"], 21, True, False),
+            (18, ["Test table", "Test table 2"], 40, True, False),
+            (19, ["Test table", "Test table 2"], 41, True, False),
+            (21, ["Test table", "Test col 2", "Test col 3"], 50, True, False),
+            (22, ["Test table", "Test col 2", "Test col 3"], 51, True, False),
+            (19, ["Test table"], 20, False, False),
+            (20, ["Test table"], 21, False, False),
+            (25, ["Test table", "Test table 2"], 40, False, False),
+            (26, ["Test table", "Test table 2"], 41, False, False),
+            (25, ["Test table", "Test col 2", "Test col 3"], 50, False, False),
+            (26, ["Test table", "Test col 2", "Test col 3"], 51, False, False),
+            (18, ["Test table"], 20, False, True),
+            (19, ["Test table"], 21, False, True),
+            (23, ["Test table", "Test table 2"], 40, False, True),
+            (24, ["Test table", "Test table 2"], 41, False, True),
+            (22, ["Test table", "Test col 2", "Test col 3"], 50, False, True),
+            (23, ["Test table", "Test col 2", "Test col 3"], 51, False, True),
+        ],
+    )
+    def test_min_table_width(
+        self, loops, fields, desired_width, border, internal_border
+    ):
+        for col_width in range(loops):
+            x = prettytable.PrettyTable()
+            x.border = border
+            x.preserve_internal_border = internal_border
+            x.field_names = fields
+            x.add_row(["X" * col_width] + ["" for _ in range(len(fields) - 1)])
+            x.min_table_width = desired_width
+            t = x.get_string()
+            if border is False and internal_border is False:
+                assert [len(x) for x in t.split("\n")] == [desired_width, desired_width]
+            elif border is False and internal_border is True:
+                assert [len(x) for x in t.split("\n")] == [
+                    desired_width,
+                    desired_width - 1,
+                    desired_width,
+                ]
+            else:
+                assert [len(x) for x in t.split("\n")] == [
+                    desired_width,
+                    desired_width,
+                    desired_width,
+                    desired_width,
+                    desired_width,
+                ]
+
+
 class TestMaxTableWidth:
     def test_max_table_width(self):
         pt = PrettyTable()
@@ -1921,6 +1982,50 @@ class TestMaxTableWidth:
         )
 
 
+class TestRowEndSection:
+    def test_row_end_section(self):
+        pt = PrettyTable()
+        v = 1
+        for row in range(4):
+            if row % 2 == 0:
+                pt.add_row([f"value {v}", f"value{v+1}", f"value{v+2}"], divider=True)
+            else:
+                pt.add_row([f"value {v}", f"value{v+1}", f"value{v+2}"], divider=False)
+            v += 3
+        pt.del_row(0)
+        assert (
+            pt.get_string().strip()
+            == """
++----------+---------+---------+
+| Field 1  | Field 2 | Field 3 |
++----------+---------+---------+
+| value 4  |  value5 |  value6 |
+| value 7  |  value8 |  value9 |
++----------+---------+---------+
+| value 10 | value11 | value12 |
++----------+---------+---------+
+""".strip()
+        )
+
+
+class TestClearing:
+    def test_clear_rows(self, row_prettytable: PrettyTable):
+        t = helper_table()
+        t.add_row(["a", "b", "c"], divider=True)
+        t.clear_rows()
+        assert t.rows == []
+        assert t.dividers == []
+        assert t.field_names == ["Field 1", "Field 2", "Field 3"]
+
+    def test_clear(self, row_prettytable: PrettyTable):
+        t = helper_table()
+        t.add_row(["a", "b", "c"], divider=True)
+        t.clear()
+        assert t.rows == []
+        assert t.dividers == []
+        assert t.field_names == []
+
+
 class TestPreservingInternalBorders:
     def test_internal_border_preserved(self):
         pt = helper_table(3)
@@ -1990,3 +2095,50 @@ class TestPreservingInternalBorders:
 </table>
 """.strip()  # noqa: E501
         )
+
+
+class TestGeneralOutput:
+    def test_text(self):
+        t = helper_table()
+        assert t.get_formatted_string("text") == t.get_string()
+        # test with default arg, too
+        assert t.get_formatted_string() == t.get_string()
+        # args passed through
+        assert t.get_formatted_string(border=False) == t.get_string(border=False)
+
+    def test_csv(self):
+        t = helper_table()
+        assert t.get_formatted_string("csv") == t.get_csv_string()
+        # args passed through
+        assert t.get_formatted_string("csv", border=False) == t.get_csv_string(
+            border=False
+        )
+
+    def test_json(self):
+        t = helper_table()
+        assert t.get_formatted_string("json") == t.get_json_string()
+        # args passed through
+        assert t.get_formatted_string("json", border=False) == t.get_json_string(
+            border=False
+        )
+
+    def test_html(self):
+        t = helper_table()
+        assert t.get_formatted_string("html") == t.get_html_string()
+        # args passed through
+        assert t.get_formatted_string("html", border=False) == t.get_html_string(
+            border=False
+        )
+
+    def test_latex(self):
+        t = helper_table()
+        assert t.get_formatted_string("latex") == t.get_latex_string()
+        # args passed through
+        assert t.get_formatted_string("latex", border=False) == t.get_latex_string(
+            border=False
+        )
+
+    def test_invalid(self):
+        t = helper_table()
+        with pytest.raises(ValueError):
+            t.get_formatted_string("pdf")
